// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "HurtHazard.hpp"

using namespace Dawn;

HurtHazard::HurtHazard(SceneItem* item) : 
  trigger(nullptr),
  SceneItemComponent(item)
{
}

void HurtHazard::onStart() {
  this->evtTriggerEnter = [&]{};

  useEffect([&]{
    this->evtTriggerEnter();
    if(this->trigger == nullptr) return;
    
    this->evtTriggerEnter = useEvent([&](CharacterController2D *controller) {
      // Check faction(s)
      auto otherFaction = controller->item->getComponent<EntityFaction>();
      if(otherFaction != nullptr && faction == otherFaction->faction) return;

      // Check health
      auto otherHealth = controller->item->getComponent<EntityHealth>();
      if(otherHealth == nullptr) return;

      // Damage
      if(otherHealth->isInvincible()) return;

      glm::vec2 back = (
        physics3Dto2D(this->transform->getWorldPosition()) -
        physics3Dto2D(controller->transform->getWorldPosition())
      );
      controller->velocity = back * -hitKnockback;
      otherHealth->damage({
        .amount = damage
      });

      std::cout << "Trigger" << std::endl;
    }, this->trigger->eventTriggerEnter);
  }, this->trigger)();
}