// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntitySwordAttack.hpp"
#include "prefabs/SwordHitbox.hpp"

using namespace Dawn;

EntitySwordAttack::EntitySwordAttack(SceneItem* item) :
  SceneItemComponent(item),
  attackTime(0.0f)
{
  
}

void EntitySwordAttack::onStart() {
  attackHitbox = SwordHitbox::create(getScene());
  attackHitbox->transform.setLocalPosition(glm::vec3(999999999, 0, 0));
  attackHitbox->transform.setParent(this->transform);

  useEvent([&](float_t delta) {
    if(this->attackTime > 0.0f) {
      this->attackTime = this->attackTime - delta;
    }
  }, getScene()->eventSceneUpdate);

  useEffect([&]{
    if(attackTime <= 0) {
      attackHitbox->transform.setLocalPosition(glm::vec3(999999999, 0, 0));
    } else {
      attackHitbox->transform.setLocalPosition(this->attackOffset);
    }
  }, attackTime);
}

void EntitySwordAttack::attack() {
  auto entityFaction = this->item->getComponent<EntityFaction>();
  auto hazard = attackHitbox->getComponent<HurtHazard>();
  this->attackTime = swingTime;
  if(hazard != nullptr && entityFaction != nullptr) {
    hazard->faction = entityFaction->faction;
  }
}

bool_t EntitySwordAttack::isAttacking() {
  return this->attackTime > 0.0f;
}